/**************************************************************************/
/*CREAR TAULA LOG 
/*PROCEDIMENT - ADDLOG
/*Crear taula LOG i el procediment ADDLOG per omplir-la              
/**************************************************************************/
CREATE TABLE LOGTABLE(
    idLog number generated always as identity primary key,
    procedureName varchar2(40) not null,
    dateExecution date default sysdate not null,
    newValue varchar2(500) not null,
    output varchar2(500) not null
);
CREATE OR REPLACE PROCEDURE ADDLOG (PROCNAME LOGTABLE.PROCEDURENAME%TYPE,
                                    NEWVALUE LOGTABLE.NEWVALUE%TYPE,
                                    OUTPUT LOGTABLE.OUTPUT%TYPE)
IS
BEGIN                        
    INSERT INTO LOGTABLE(PROCEDURENAME, NEWVALUE, OUTPUT) 
        VALUES(PROCNAME, NEWVALUE, OUTPUT);
        
END;
/**************************************************************************/
/*PROCEDIMENT - CREATEBATCHPERSON
/*Procediment per crear persones massivament, aix es genera una quantitat
/*adequada de dades per poder operar amb les altres taules               
/**************************************************************************/
create or replace procedure createBatchPerson( 
    ININUMBER NUMBER,
    RSP OUT VARCHAR)
IS
    NUMBERID NUMBER;
    IDpERSON VARCHAR2(15);
    NIFPERSON VARCHAR2(10);
    param varchar(500);
BEGIN
    RSP:='OK';
    NUMBERID:=ININUMBER - 1;
    param:=ininumber;
    FOR I IN 1..10
    LOOP      
        NUMBERID:=NUMBERID + 1;
        IDPERSON := 'P'||NUMBERID;
        NIFPERSON := 'NIF'||NUMBERID;
        INSERT INTO PERSON(IDPERSON, NIF, NAME, SURNAME, BIRTHDATE, GENDER) VALUES(IDPERSON, NIFPERSON, 'Michael', 'Scott', '01-01-2000', 'Male');
        NUMBERID:=NUMBERID + 1;
        IDPERSON := 'P'||NUMBERID;
        NIFPERSON := 'NIF'||NUMBERID;
        INSERT INTO PERSON(IDPERSON, NIF, NAME, SURNAME, BIRTHDATE, GENDER) VALUES(IDPERSON, NIFPERSON, 'Lara', 'Scott', '01-02-2000', 'Female');
        NUMBERID:=NUMBERID + 1;
        IDPERSON := 'P'||NUMBERID;
        NIFPERSON := 'NIF'||NUMBERID;
        INSERT INTO PERSON(IDPERSON, NIF, NAME, SURNAME, BIRTHDATE, GENDER) VALUES(IDPERSON, NIFPERSON, 'Rey', 'Hitchcock', '01-03-2000', 'Other');
    END LOOP;
    ADDLOG($$PLSQL_UNIT,param,RSP);
EXCEPTION
WHEN OTHERS THEN
    RSP:='ERROR'|| ':' ||SQLERRM;
    ADDLOG($$PLSQL_UNIT,param,RSP);
END;
/**************************************************************************/
/*PROCEDIMENT - MODIFYPERSON
/*modifica persona
/**************************************************************************/
create or replace procedure MODIFYPERSON( 
    i_idperson person.idperson%type,
    i_nif person.nif%type,
    i_name person.name%type,
    i_surname person.surname%type,
    i_birthdate person.birthdate%type,
    i_gender  person.gender%type,
    RSP OUT VARCHAR)
IS
    param varchar(500);
BEGIN
    RSP:='OK';
    param:= i_idperson || i_nif || i_name || i_surname || i_birthdate || i_gender;

    UPDATE person 
    SET nif = i_nif,
        name = i_name,
        surname = i_surname,
        birthdate = i_birthdate,
        gender = i_gender
    WHERE idperson = i_idperson;
   
    ADDLOG($$PLSQL_UNIT,param,RSP);
EXCEPTION
WHEN OTHERS THEN
    RSP:='ERROR'|| ':' ||SQLERRM;
    ADDLOG($$PLSQL_UNIT,param,RSP);
END;
/**************************************************************************/
/*PROCEDIMENT - DROPPERSON
/*elimina persona
/**************************************************************************/
create or replace procedure dropPerson( 
    i_idperson person.idperson%type,
    RSP OUT VARCHAR)
IS
    param varchar(500);
BEGIN
    RSP:='OK';
    param:= i_idperson;
    DELETE FROM PERSON WHERE IDPERSON = I_IDPERSON;
    ADDLOG($$PLSQL_UNIT,param,RSP);
EXCEPTION
WHEN OTHERS THEN
    RSP:='ERROR'|| ':' ||SQLERRM;
    ADDLOG($$PLSQL_UNIT,param,RSP);
END;
/**************************************************************************/
/*PROCEDIMENT - CREATECHANGEMANAGER
/*Procediment per assignar el gestor de canvis de l'empresa
/*Noms n'hi pot haver un, aix que cal comprovar que no existeix
/**************************************************************************/
CREATE OR REPLACE PROCEDURE createchangemanager (
    changemanager person.idperson%TYPE, substitute person.idperson%TYPE, rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    howmany NUMBER;
    cm_exists EXCEPTION;
    cm_substitute_equal EXCEPTION;
BEGIN
    rsp := 'OK';
    param := changemanager || substitute;
    --Comprovem que el gestor de canvis i el substitut siguin diferents
    IF changemanager = substitute THEN
        RAISE cm_substitute_equal;
    END IF;
    --Revisem si existeix una entrada per change manager a membergcab
    SELECT COUNT(*)
    INTO howmany
    FROM membergcab
    WHERE rolegcab = 'CM';
    --existeix un change manager, cal mostrar excepci
    IF howmany > 0 THEN
        RAISE cm_exists;
    --no existeix change manager, es crea el registre a membergcab
    ELSE
        INSERT INTO membergcab (
            idmember, rolegcab, substitute
        ) VALUES (
            changemanager, 'CM', substitute
        );

        addlog($$plsql_unit, param, rsp);
    END IF;

EXCEPTION
    WHEN cm_substitute_equal THEN
        rsp := 'ERROR' || ':' || 'El substitut es igual que el gestor de canvis';
        addlog($$plsql_unit, param, rsp);
    WHEN cm_exists THEN
        rsp := 'ERROR' || ':' || 'Ja hi ha un gestor de canvis';
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - CREATEBATCHMEMBERGCAB
/*Procediment per assignar massivament persones com a membres del GCAB              
/**************************************************************************/
CREATE OR REPLACE PROCEDURE createbatchmembergcab (
    ininumber NUMBER, rsp OUT VARCHAR
) IS

    numberid NUMBER;
    idsubstitute VARCHAR2(15);
    idmember VARCHAR2(15);
    param VARCHAR(500);
BEGIN
    rsp := 'OK';
    param := ininumber;
    numberid := ininumber - 1;
    FOR i IN 1..5 LOOP
        numberid := numberid + 1;
        idmember := 'P' || numberid;
        numberid := numberid + 1;
        idsubstitute := 'P' || numberid;
        --inserir Technical Manager i substitut
        INSERT INTO membergcab (
            idmember, rolegcab, substitute
        ) VALUES (
            idmember, 'TM', idsubstitute
        );

        numberid := numberid + 1;
        idmember := 'P' || numberid;
        numberid := numberid + 1;
        idsubstitute := 'P' || numberid;
        --inserir Business Manager i substitut
        INSERT INTO membergcab (
            idmember, rolegcab, substitute
        ) VALUES (
            idmember, 'BM', idsubstitute
        );

    END LOOP;

    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - MODIFYMEMBERGCAB
/*Procediment modificar membres del GCAB              
/**************************************************************************/
CREATE OR REPLACE PROCEDURE modifyMembergcab (
    i_idmember membergcab.idmember%type,
    i_rolegcab membergcab.rolegcab%type,
    i_substitute membergcab.substitute%type,
    rsp OUT VARCHAR
) IS

    param VARCHAR(500);
BEGIN
    rsp := 'OK';
    param := i_idmember || i_rolegcab || i_substitute;
    UPDATE membergcab 
    SET rolegcab = i_rolegcab,
        substitute = i_substitute
    WHERE idmember = i_idmember;

    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - DROPMEMBERGCAB
/*Procediment eliminar membres del GCAB              
/**************************************************************************/
CREATE OR REPLACE PROCEDURE dropMemberGCAB (
    i_idmember membergcab.idmember%type,
    rsp OUT VARCHAR
) IS

    param VARCHAR(500);
BEGIN
    rsp := 'OK';
    param := i_idmember;  
    DELETE FROM MEMBERGCAB WHERE IDMEMBER = I_IDMEMBER;
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - CREATEAPP
/*Procediment per crear aplicacions:  
/*Comprova que els responsables tcnics i de negoci existeixin, calcula
/*l'atribut derivat impactoos i crea el registre a AppInventory
/**************************************************************************/
CREATE OR REPLACE PROCEDURE createapp (
    idapp appinventory.idapp%TYPE, appname appinventory.appname%TYPE, datepro appinventory.datepro%TYPE, technology appinventory.technology%TYPE
    , infraestructure appinventory.infraestructure%TYPE, numusers appinventory.numusers%TYPE, criticality appinventory.criticality%TYPE
    , technicalresp appinventory.technicalresp%TYPE, businessresp appinventory.businessresp%TYPE, europe geoareaapp.geoarea%TYPE, asia
    geoareaapp.geoarea%TYPE, oceania geoareaapp.geoarea%TYPE, america geoareaapp.geoarea%TYPE, rsp OUT VARCHAR
) IS

    impactoos appinventory.impactoos%TYPE;
    param VARCHAR(500);
    idtechm membergcab.idmember%TYPE;
    idbusm membergcab.idmember%TYPE;
    infoselect VARCHAR(30);
    no_geo_area EXCEPTION;
BEGIN
    rsp := 'OK';
    param := idapp || appname || datepro || technology || infraestructure
             || numusers || criticality || technicalresp || businessresp
             || europe || asia || oceania || america;
             
    --Comprovar que s'ha introduit una rea geogrfica vlida
    IF (
        europe != 'Europa' AND asia != 'Asia' AND oceania != 'Oceania' AND america != 'America'
    ) THEN
        RAISE no_geo_area;
    END IF;

    --comprovar technical manager i business manager existeixen
    infoselect := 'Responsable Tcnic'; --info per l'error
    SELECT idmember
    INTO idtechm
    FROM membergcab
    WHERE idmember = technicalresp AND rolegcab = 'TM';

    infoselect := 'Responsable de negoci'; --info per l'error
    SELECT idmember
    INTO idbusm
    FROM membergcab
    WHERE idmember = businessresp AND rolegcab = 'BM';
    
    --calcular atribut derivat impactOOS
    IF numusers < 50000 AND criticality <= 4 THEN
        impactoos := 'Low';
    ELSIF ( 50000 <= numusers AND numusers <= 100000 AND criticality <= 8) OR 
        ( 4 < criticality AND criticality <= 8 AND numusers <= 100000) THEN
        impactoos := 'Medium';
    ELSIF numusers > 100000 OR criticality > 8 THEN
        impactoos := 'High';
    END IF;

    --insertar registre a AppInventory
    INSERT INTO appinventory (
        idapp, appname, datepro, technology, infraestructure, numusers, criticality, impactoos, technicalresp, businessresp
    ) VALUES (
        idapp, appname, datepro, technology, infraestructure, numusers, criticality, impactoos, technicalresp, businessresp
    );
    
    --insertar rea geogrfica (GeoAreaApp)
    --insertar europa
    IF europe = 'Europa' THEN
        INSERT INTO geoareaapp (
            idapp, geoarea
        ) VALUES (
            idapp, europe
        );

    END IF;

    --insertar asia
    IF asia = 'Asia' THEN
        INSERT INTO geoareaapp (
            idapp, geoarea
        ) VALUES (
            idapp, asia
        );

    END IF;

    IF oceania = 'Oceania' THEN
        INSERT INTO geoareaapp (
            idapp, geoarea
        ) VALUES (
            idapp, oceania
        );

    END IF;

    IF america = 'America' THEN
        INSERT INTO geoareaapp (
            idapp, geoarea
        ) VALUES (
            idapp, america
        );

    END IF;
    
    --registrar procediment al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN no_data_found THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        dbms_output.put_line('ERROR: El ' || infoselect || ' no existeix');
    WHEN no_geo_area THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        dbms_output.put_line('ERROR: Introdueix una rea geogrfica vlida');
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - MODIFYAPP
/**************************************************************************/
CREATE OR REPLACE PROCEDURE modifyApp (
  i_idapp appinventory.idapp%type, 
  i_appname appinventory.appname%type, 
  I_DATEPRO appinventory.datepro%type,
  i_tech appinventory.technology%type,
  i_infra appinventory.infraestructure%type,
  i_num appinventory.numusers%type,
  i_criticality appinventory.criticality%type,
  i_tm appinventory.technicalresp%type,
  i_bm appinventory.businessresp%type,
  i_eur geoareaapp.geoarea%TYPE,
  i_asia geoareaapp.geoarea%TYPE, 
  i_oce geoareaapp.geoarea%TYPE, 
  i_amer geoareaapp.geoarea%TYPE,
  rsp OUT VARCHAR
) IS
    I_impactoos appinventory.impactoos%TYPE;
    param VARCHAR(500);
    idtechm membergcab.idmember%TYPE;
    idbusm membergcab.idmember%TYPE;
    infoselect VARCHAR(30);
    no_geo_area EXCEPTION;
BEGIN
    rsp := 'OK';
    param := I_idapp || i_appname || i_datepro || i_tech || i_infra
             || i_num || i_criticality || i_tm || i_bm ||i_eur || i_asia || i_oce || i_amer;  
             
    IF (i_eur != 'Europa' AND i_asia != 'Asia' AND i_oce != 'Oceania' AND i_amer != 'America'
    ) THEN
        RAISE no_geo_area;
    END IF;

    --comprovar technical manager i business manager existeixen
    infoselect := 'Responsable Tcnic'; --info per l'error
    SELECT idmember
    INTO idtechm
    FROM membergcab
    WHERE idmember = I_TM AND rolegcab = 'TM';

    infoselect := 'Responsable de negoci'; --info per l'error
    SELECT idmember
    INTO idbusm
    FROM membergcab
    WHERE idmember = I_BM AND rolegcab = 'BM';
   
    --calcular atribut derivat impactOOS
    IF I_num < 50000 AND I_criticality <= 4 THEN
        I_impactoos := 'Low';
    ELSIF ( 50000 <= I_num AND I_num <= 100000 AND I_criticality <= 8) OR 
        ( 4 < I_criticality AND I_criticality <= 8 AND I_num <= 100000) THEN
        I_impactoos := 'Medium';
    ELSIF I_num > 100000 OR I_criticality > 8 THEN
        I_impactoos := 'High';
    END IF;
    
    UPDATE APPINVENTORY
    SET APPNAME = i_APPNAME,
        datepro = i_datepro,
        technology = i_tech,
        infraestructure = i_infra,
        numusers = i_num,
        criticality = i_criticality,
        impactoos = i_impactOOS,
        technicalresp = i_tm,
        businessresp = i_bm
    WHERE idapp = I_idapp;    
    
    delete from geoareaApp where idapp = i_idapp;
    
    if i_eur = 'Europa' then
        INSERT INTO geoareaapp (idapp, geoarea) VALUES ( i_idapp, i_eur );
    END IF;
    --insertar asia
    IF i_asia = 'Asia' THEN
        INSERT INTO geoareaapp (idapp, geoarea) VALUES (i_idapp, i_asia);
    END IF;

    IF i_oce = 'Oceania' THEN
        INSERT INTO geoareaapp (idapp, geoarea ) VALUES (i_idapp, i_oce);
    END IF;

    IF i_amer = 'America' THEN
        INSERT INTO geoareaapp (idapp, geoarea) VALUES (i_idapp, i_amer);
    END IF;
    --registrar procediment al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN no_data_found THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        dbms_output.put_line('ERROR: El ' || infoselect || ' no existeix');
    WHEN no_geo_area THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        dbms_output.put_line('ERROR: Introdueix una rea geogrfica vlida');
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - DROPAPP
/**************************************************************************/
CREATE OR REPLACE PROCEDURE DROPApp (
  i_idapp appinventory.idapp%type, 
  rsp OUT VARCHAR
) IS
    I_impactoos appinventory.impactoos%TYPE;
    param VARCHAR(500);
    idtechm membergcab.idmember%TYPE;
    idbusm membergcab.idmember%TYPE;
    infoselect VARCHAR(30);
    no_geo_area EXCEPTION;
BEGIN
    rsp := 'OK';
    param := I_idapp ;  
    delete from geoareaapp where idapp = i_idapp;
    delEte from appinventory where idapp = i_idapp;
    --registrar procediment al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/***************************************************************************/
/*PROCEDIMENT - INCREASECOUNTERS
/*Procediment auxiliar pel manteniment de la taula estadstica Counters
/*************************************************************************/
CREATE OR REPLACE PROCEDURE increasecounters (
    i_idcounter counters.idcounter%TYPE, i_yearinfo counters.yearinfo%TYPE, rsp OUT VARCHAR
) IS
    v_exists NUMBER;
    v_newvalue NUMBER;
    v_oldvalue NUMBER;
    param VARCHAR(500);
BEGIN
    rsp := 'OK';
    param := i_idcounter || i_yearinfo;
    SELECT COUNT(*)
    INTO v_exists
    FROM counters
    WHERE idcounter = i_idcounter AND yearinfo = i_yearinfo;
    --si el concepte (idcounters) no existeix, es crea la lnea amb valor 1
    IF v_exists = 0 THEN
        v_newvalue := 1;
        INSERT INTO counters (
            idcounter, yearinfo, totalnumber
        ) VALUES (
            i_idcounter, i_yearinfo, v_newvalue
        );

    ELSE
          UPDATE counters
            SET
                totalnumber = totalnumber + 1
            WHERE idcounter = i_idcounter AND yearinfo = i_yearinfo;

    END IF;

    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/***************************************************************************/
/*PROCEDIMENT - DECREASECOUNTERS
/*Procediment auxiliar pel manteniment de la taula estadstica Counters
/*Es crida en la creaci de canvis i altres procediments
/*************************************************************************/
CREATE OR REPLACE PROCEDURE decreasecounters (
    i_idcounter counters.idcounter%TYPE, i_yearinfo counters.yearinfo%TYPE, rsp OUT VARCHAR
) IS
    v_exists NUMBER;
    v_newvalue NUMBER;
    v_oldvalue NUMBER;
    param VARCHAR(500);
    not_exists EXCEPTION;
BEGIN
    rsp := 'OK';
    param := i_idcounter || i_yearinfo;
    SELECT COUNT(*)
    INTO v_exists
    FROM counters
    WHERE idcounter = i_idcounter AND yearinfo = i_yearinfo;
    --si el concepte (idcounters) no existeix, es mostra error
    IF v_exists = 0 THEN
        RAISE not_exists;
    ELSE
        SELECT totalnumber
        INTO v_oldvalue
        FROM counters
        WHERE idcounter = i_idcounter AND yearinfo = i_yearinfo;
        --si el valor s 0, es mostra error
        IF v_oldvalue = 0 THEN
            RAISE not_exists;
        END IF;
        UPDATE counters
        SET totalnumber = totalnumber - 1
        WHERE idcounter = i_idcounter AND yearinfo = i_yearinfo;
    END IF;

    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN not_exists THEN
        rsp := 'ERROR: el concepte no existeix per l''any introduit o s 0';
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - increaseCurrChangesTM
/*Procediment auxiliar per registrar els canvis en curs de cada TM
/**************************************************************************/
CREATE OR REPLACE PROCEDURE increaseCurrChangesTM (
    i_idapp AppInventory.idApp%type,
    rsp out varchar
) IS
    param VARCHAR(500);
    v_idtm membergcab.idmember%type;
    v_exists NUMBER;
    v_newvalue NUMBER;
    v_oldvalue NUMBER;
BEGIN
    param := i_idapp;
    rsp:= 'OK';
    --busquem el TM de l'aplicaci
    select technicalresp into v_idtm
    from appinventory where idapp = i_idapp;
    
    SELECT COUNT(*)
    INTO v_exists
    FROM technicalManagerData
    WHERE technicalmanager = v_idtm;

    --si no hi ha info pel TM tractat, es crea la lnea amb valor 1
    IF v_exists = 0 THEN
        v_newvalue := 1;
        INSERT INTO TechnicalManagerData (
            technicalmanager, correctexecutions, currchanges
        ) VALUES (
            v_idtm, 0, v_newvalue
        );
    -- Si el TM ja t dades a la taula, s'incremente +1 el valor de currchanges
    ELSE
        UPDATE TechnicalManagerData
        SET currchanges = currchanges + 1
         WHERE technicalmanager = v_idtm;
    END IF;
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - decreaseCurrChangesTM
/*Procediment auxiliar per registrar els canvis en curs de cada TM
/**************************************************************************/
CREATE OR REPLACE PROCEDURE decreaseCurrChangesTM (
    i_idapp AppInventory.idApp%type,
    rsp out varchar
) IS
    param VARCHAR(500);
    v_idtm membergcab.idmember%type;
    v_exists NUMBER;
    v_newvalue NUMBER;
    v_oldvalue NUMBER;
    not_exists exception;
BEGIN
    param := i_idapp;
    rsp:= 'OK';
    --busquem el TM de l'aplicaci
    select technicalresp into v_idtm
    from appinventory where idapp = i_idapp;
    
    SELECT COUNT(*)
    INTO v_exists
    FROM technicalManagerData
    WHERE technicalmanager = v_idtm;

    --si no hi ha info pel TM tractat, salta excepci
    IF v_exists = 0 THEN
        RAISE not_exists;
    ELSE
        SELECT currchanges
        INTO v_oldvalue
        FROM TechnicalManagerData
        WHERE technicalmanager = v_idtm;

        IF v_oldvalue = 0 THEN
            RAISE not_exists;
        END IF;
        UPDATE TechnicalManagerData
        SET currchanges = currchanges - 1
         WHERE technicalmanager = v_idtm;

    END IF;
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN not_exists THEN
        rsp := 'ERROR: El TM no t canvis en curs';
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/***************************************************************************/
/*PROCEDIMENT - CALCUL_GEOSCOPE_CATEGORY
/*Procediment auxiliar pel clcul de l'abast geogrfic i categoria del canvi
/*************************************************************************/
CREATE OR REPLACE PROCEDURE calcul_geoscope_category (
    idapplication appinventory.idapp%TYPE, geoscope OUT changes.geoscope%TYPE, changecategory OUT changes.category%TYPE
) IS

    param VARCHAR(500);
    rsp VARCHAR(500);
    numgeoarea NUMBER;
    impactoos appinventory.impactoos%TYPE;
BEGIN
    rsp := 'OK';
    param := idapplication;
    SELECT COUNT(*)
    INTO numgeoarea
    FROM geoareaapp
    WHERE idapp = idapplication;

    IF numgeoarea = 0 THEN
        RAISE no_data_found;
    ELSIF numgeoarea = 1 THEN
        geoscope := 'Low';
    ELSIF numgeoarea = 2 THEN
        geoscope := 'Medium';
    ELSE
        geoscope := 'High';
    END IF;

    IF geoscope = 'High' THEN
        changecategory := '1';
    ELSE
        SELECT impactoos
        INTO impactoos
        FROM appinventory
        WHERE idapp = idapplication;

        IF impactoos = 'High' OR geoscope = 'High' THEN
            changecategory := '1';
        ELSIF
            impactoos = 'Low' AND geoscope = 'Low'
        THEN
            changecategory := '3';
        ELSE
            changecategory := '2';
        END IF;

    END IF;

    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN no_data_found THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
        dbms_output.put_line('ERROR: Aplicaci incorrecta');
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - CREATECHANGECATEGORY
/*Procediment per crear categories de canvi            
/**************************************************************************/
CREATE OR REPLACE PROCEDURE createChangeCategory (
    i_importance changecategory.importance%type,
    i_creationDate changecategory.creationDate%type,
    i_datein changecategory.datein%type,
    i_dateout changecategory.dateout %type,
      rsp OUT VARCHAR
) IS

    param VARCHAR(500);
BEGIN
    rsp := 'OK';
    param :=  i_importance || i_creationDate || i_datein || i_dateout;
            
    INSERT INTO changecategory (importance, creationdate, datein, dateout
        ) VALUES ( i_importance, i_creationDate, i_datein, i_dateout);

    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - MODIFYCHANGECATEGORY
/*Procediment per modificar categories de canvi            
/**************************************************************************/
CREATE OR REPLACE PROCEDURE modifyChangeCategory (
    i_importance changecategory.importance%type,
    i_creationDate changecategory.creationDate%type,
    i_datein changecategory.datein%type,
    i_dateout changecategory.dateout %type,
      rsp OUT VARCHAR
) IS

    param VARCHAR(500);
BEGIN
    rsp := 'OK';
    param :=  i_importance || i_creationDate || i_datein || i_dateout;
    
    UPDATE changeCategory
    SET creationDate = i_creationDate,
        dateIn = i_dateIn,
        dateOut = i_dateout
    WHERE importance = i_importance;

    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - DROPCHANGECATEGORY
/*Procediment per modificar categories de canvi            
/**************************************************************************/
CREATE OR REPLACE PROCEDURE dropChangeCategory (
    i_importance changecategory.importance%type,
      rsp OUT VARCHAR
) IS

    param VARCHAR(500);
BEGIN
    rsp := 'OK';
    param :=  i_importance ;
    
    DELETE FROM changecategory where importance = i_importance;

    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - CREATECHANGE
/*Procediment per crear canvis            
/**************************************************************************/
CREATE OR REPLACE PROCEDURE createchange (
    idchange changes.idchange%TYPE, name changes.name%TYPE, description changes.description%TYPE, 
    application changes.application%TYPE
    , rsp OUT VARCHAR
) IS

    param VARCHAR(500);
    idapplication appinventory.idapp%TYPE;
    geoscope changes.geoscope%TYPE;
    category changes.category%TYPE;
BEGIN
    rsp := 'OK';
    param := idchange || name || description || application;
    --Es calculen l'abast geogrfic i la categoria
    calcul_geoscope_category(application, geoscope, category);
    --es crea el registre a la taula changes
    INSERT INTO changes (
        idchange, name, description, geoscope, category, application, statuschange
    ) VALUES (
        idchange, name, description, geoscope, category, application, 'PA'
    );
    --s'incrementa el valor de canvis en procs d'aprovaci a la taula estadstica counters
    increasecounters('PA', '9999', rsp);
    --s'incrementa el valor de canvis en curs pel responsable tcnic de l'app
    increaseCurrChangesTM(application, rsp);
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - MODIFYCHANGE
/*Procediment per modificar canvis            
/**************************************************************************/
CREATE OR REPLACE PROCEDURE modifychange (
    idcanvi changes.idchange%TYPE, newname changes.name%TYPE, newdescription changes.description%TYPE, newapp changes.application%TYPE
    , rsp OUT VARCHAR
) IS

    param VARCHAR(500);
    newcategory changes.category%TYPE;
    newgeoscope changes.geoscope%TYPE;
    oldapp changes.application%TYPE;
BEGIN
    rsp := 'OK';
    param := idcanvi || newname || newdescription || newapp;
    
    --comprovem que el canvi existeix i seleccionem si l'app canvia
    SELECT application
    INTO oldapp
    FROM changes
    WHERE idchange = idcanvi;

    --si l'app canvia
    IF oldapp != newapp THEN
        -- es torna a calcular el l'abast i la categoria del canvi
        calcul_geoscope_category(newapp, newgeoscope, newcategory);
        --s'actualitzen els canvis en curs pel TM nou (+1) i vell (-1)
        increaseCurrChangesTM(newapp, rsp);
        decreaseCurrChangestm(oldapp, rsp);
    END IF;
    
    --actualitzar el canvi
    UPDATE changes
    SET name = newname, description = newdescription, application = newapp, geoscope = newgeoscope, category = newcategory
    WHERE idchange = idcanvi;

    --afegir entrada al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - DROPCHANGE
/*Procediment per eliminar canvis            
/**************************************************************************/
CREATE OR REPLACE PROCEDURE dropchange (
    i_idchange changes.idchange%TYPE, rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_statuschange changes.statuschange%TYPE;
    v_app changes.application%type;
    change_approved_rejected EXCEPTION;
BEGIN
    rsp := 'OK';
    param := i_idchange;
    --noms es poden eliminar canvis pendents d'aprovar
    SELECT statuschange, application
    INTO v_statuschange, v_app
    FROM changes
    WHERE idchange = i_idchange;

    IF v_statuschange != 'PA' THEN
        RAISE change_approved_rejected;
    END IF;
    --s'elimina el canvi
    DELETE FROM changes
    WHERE idchange = i_idchange;
    --es redueix el valor de canvis pendents d'aprovaci a Counters
    decreasecounters('PA', '9999', rsp);
    --es redueixen -1 els canvis en curs del TM de l'app
    decreaseCurrChangesTM(v_app, rsp);
    --Registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN change_approved_rejected THEN
        rsp := 'ERROR: El canvi est aprovat o rebutjat i no es pot eliminar';
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;

/**************************************************************************/
/*PROCEDIMENT - MODIFYSTATUSCHANGE
/*Procediment auxiliar per modificar l'estat del canvi   
/**************************************************************************/
CREATE OR REPLACE PROCEDURE modifystatuschange (
    i_idchange changes.idchange%TYPE, i_newstatus changes.statuschange%TYPE, rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_oldstatus changes.statuschange%TYPE;
    reject_not_possible EXCEPTION;
BEGIN
    rsp := 'OK';
    param := i_idchange || i_newstatus;
    
    --s'obt l'estat anterior del canvi
    SELECT statuschange
    INTO v_oldstatus
    FROM changes
    WHERE idchange = i_idchange;
    
    --un canvi aprovat no es pot rebutjar
    IF
        i_newstatus = 'RE' AND v_oldstatus = 'AP'
    THEN
        RAISE reject_not_possible;
    END IF;
    
    --actualitzar el canvi
    UPDATE changes
    SET
        statuschange = i_newstatus
    WHERE idchange = i_idchange;

    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN reject_not_possible THEN
        rsp := 'ERROR: un canvi aprovat no es pot rebutjar';
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;

/**************************************************************************/
/*PROCEDIMENT - REJECTCHANGE
/*Procediment per rebutjar un canvi   
/**************************************************************************/
CREATE OR REPLACE PROCEDURE rejectchange (
    i_idchange changes.idchange%TYPE, i_yearrejection counters.yearinfo%TYPE, rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    rsp_aux VARCHAR(200);
    v_app changes.application%type;
BEGIN
    param := i_idchange || i_yearrejection;
    --modificar l'estat del canvi
    modifystatuschange(i_idchange, 'RE', rsp);
    --actualitzar nombre canvis rebutjats per any proporcionat
    increasecounters('RE', i_yearrejection, rsp_aux);
    --reduir nombre de canvis en procs d'aprovaci
    decreasecounters('PA', '9999', rsp_aux);
    --reduir nombre canvis en curs pel responsable tcnic
    select application into v_app
    from changes where idchange = i_idchange;
    decreaseCurrChangesTM(v_app, rsp);
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - APPROVECHANGE
/*Procediment per registrar les aprovacions 
/**************************************************************************/
CREATE OR REPLACE PROCEDURE approvechange (
    i_idapp appinventory.idapp%type,    
    i_idchange changes.idchange%TYPE, 
    i_idperson membergcab.idmember%TYPE, 
    i_date approval.dateapproval%TYPE, i_commentary approval.commentary%TYPE
    , rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    rsp_aux VARCHAR(200);
    v_role membergcab.rolegcab%TYPE;
    v_roleaux membergcab.rolegcab%type;
    v_error varchar(100);
    v_techresp appinventory.technicalresp%type;
    v_busresp appinventory.businessresp%type;
    v_idmember membergcab.idmember%type;
BEGIN
    rsp := 'OK';
    param := i_idapp || i_idchange || i_idperson || i_date || i_commentary;
    
    --busquem el rol de l'aprovador
    v_error:= 'La persona no s membre del GCAB';
    SELECT idmember, rolegcab
    INTO v_idmember, v_role
    FROM membergcab
    WHERE ( idmember = i_idperson or substitute = i_idperson );
    
    --si l'aprovador s TM o BM, comprovar que s el TM/BM de l'app
    if v_role = 'TM' then
        v_error:= 'L''aprovador no s el responsable tcnic de l''aplicaci';
        SELECT technicalresp
        INTO v_techresp
        FROM appinventory
        WHERE idapp = i_idapp and technicalresp = v_idmember;
    elsif v_role = 'BM' THEN
        v_error:= 'L''aprovador no s el responsable de negoci de l''aplicaci';
        SELECT businessresp
        INTO v_busresp
        FROM appinventory
        WHERE idapp = i_idapp and businessresp = v_idmember;
    --El gestor de canvis s l'ltim d'aprovar.
    ELSIF v_role = 'CM' THEN
        --Es comprova que el TM hagi aprovat
         v_error:= 'Falta l''aprovaci del TM';
        select roleapprover into v_roleaux
        from approval
        where idchange = i_idchange and roleapprover = 'TM';
        --Es comprova que el BM hagi aprovat
        v_error:= 'Falta l''aprovaci del BM';
        select roleapprover into v_roleaux
        from approval
        where idchange = i_idchange and roleapprover = 'BM';
        --es modifica l'estat del canvi a a provat
        modifystatuschange(i_idchange, 'AP', rsp_aux);
    END IF;
    
    --s'introdueix l'aprovaci a Approval
    INSERT INTO approval (
        idperson, idchange, dateapproval, roleapprover, commentary
    ) VALUES (
        i_idperson, i_idchange, i_date, v_role, i_commentary
    );
    
    --En cas que s'hagi aprovat el canvi
    if v_role = 'CM' THEN
        -- +1 valor de canvis aprovats
        increasecounters('AP', extract(year from i_date), rsp_aux);
        -- -1 canvis pendents d'aprovaci
        decreasecounters('PA', '9999', RSP_AUX);
    END IF;
    --registre al log
    addlog($$plsql_unit, param, rsp);
    
EXCEPTION
    WHEN no_data_found THEN
        rsp := 'ERROR' || ':' || v_error;
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;

/**************************************************************************/
/*PROCEDIMENT - MODIFYAPPROVAL
/*Procediment per modificar una aprovaci d'un canvi
/**************************************************************************/
 CREATE OR REPLACE PROCEDURE modifyapproval (
    i_idperson membergcab.idmember%TYPE, i_idchange changes.idchange%TYPE,
    i_newdate approval.dateapproval%TYPE, i_newcommentary approval.commentary%TYPE
    , rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_olddate approval.dateapproval%type;
BEGIN
    rsp := 'OK';
    param := i_idperson || i_idchange || i_newdate || i_newcommentary ;  
   
    update approval
    set dateapproval = i_newdate,
        commentary = i_newcommentary
    where idperson = i_idperson
        and idchange = i_idchange;
    
    --registre al log
    addlog($$plsql_unit, param, rsp);
      
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;

/**************************************************************************/
/*PROCEDIMENT - DROPAPPROVAL
/*Procediment per eliminar una aprovaci d'un canvi
/**************************************************************************/
CREATE OR REPLACE PROCEDURE dropapproval (
    i_idperson membergcab.idmember%TYPE, i_idchange changes.idchange%TYPE,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
     v_role approval.roleapprover%type;
     v_executed number;
     v_error varchar(100);
     rsp_aux VARCHAR(200);
     executed_change exception;
     v_dateapproval approval.dateapproval%type;
BEGIN
    
    rsp := 'OK';
    param := i_idperson || i_idchange;    
    
    --busquem l'aprovaci que es vol eliminar
    v_error:= 'L''aprovaci no existeix';
    SELECT dateapproval, roleapprover
    INTO v_dateapproval, v_role
    FROM approval
    where idperson = i_idperson and 
    idchange = i_idchange;
    
    --Si s'elimina l'aprovaci del gestor de canvis, el canvi deixa d'estar aprovat i 
    -- s'ha de canviar l'estat
    IF v_role = 'CM' THEN
        --si el canvi est executat, no es pot eliminar l'aprovaci
        select count(*) into v_executed
        from executedchanges
        where idchange = i_idchange;
        if v_executed > 0 then
            raise executed_change;
        else
             --es modifica l'estat del canvi a pendent d'aprovaci
            modifystatuschange(i_idchange, 'PA', rsp_aux);
        end if;
       
    END IF;
    
    --s'elimina l'aprovaci
    DELETE FROM APPROVAL WHERE IDPERSON = I_IDPERSON AND IDCHANGE = I_IDCHANGE;
    
    --si s'ha canviat l'estat del canvi, actualitzar Counters
    if v_role = 'CM' THEN
        decreasecounters('AP', extract(year from v_dateapproval), rsp_aux); -- -1 canvis aprovats
        increasecounters('PA', '9999', RSP_AUX); -- +1 canvis pendents aprovar
    END IF;
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN no_data_found THEN
        rsp := 'ERROR' || ':' || v_error;
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - INCREASECORRECTEXECUTIONSTM
/*incrementa canvis correctes a TechnicalManagerData
/**************************************************************************/
CREATE OR REPLACE PROCEDURE increaseCorrectExecutionsTM (
    i_idapp AppInventory.idApp%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_idtm membergcab.idmember%type;
    v_exists NUMBER;
    --v_newvalue NUMBER;
    v_oldvalue NUMBER;
BEGIN    
    rsp := 'OK';
    param := i_idapp ; 
    --busquem el TM de l'aplicaci
    select technicalresp into v_idtm
    from appinventory where idapp = i_idapp;
    --mirem ja existeix info a la taula pel TM tractat
    SELECT COUNT(*)
    INTO v_exists
    FROM technicalManagerData
    WHERE technicalmanager = v_idtm;
    --si no hi ha info pel TM tractat, es crea la lnea amb valor 1
    IF v_exists = 0 THEN
        INSERT INTO TechnicalManagerData (
            technicalmanager, correctexecutions, currchanges
        ) VALUES (
            v_idtm, 1, 0
        );
    -- Si el TM ja t dades a la taula, s'incremente +1 el valor de correctExecutions
    ELSE
        UPDATE TechnicalManagerData
        SET correctExecutions = correctExecutions + 1
         WHERE technicalmanager = v_idtm;
    END IF;
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - DECREASECORRECTEXECUTIONSTM
/*redueix canvis correctes a TechnicalManagerDataa
/**************************************************************************/
CREATE OR REPLACE PROCEDURE decreaseCorrectExecutionsTM (
    i_idapp AppInventory.idApp%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_idtm membergcab.idmember%type;
    v_exists NUMBER;
    v_oldvalue NUMBER;
    not_exists exception;
BEGIN    
    rsp := 'OK';
    param := i_idapp ; 
    --busquem el TM de l'aplicaci
    select technicalresp into v_idtm
    from appinventory where idapp = i_idapp;
    --mirem ja existeix info a la taula pel TM tractat
    SELECT COUNT(*)
    INTO v_exists
    FROM technicalManagerData
    WHERE technicalmanager = v_idtm;
    --si no hi ha info pel TM tractat, es crea la lnea amb valor 1
    IF v_exists = 0 THEN
        RAISE NOT_EXISTS;
    -- Si el TM ja t dades a la taula, es redueix -1 el valor de correctExecutions
    ELSE
        SELECT correctExecutions
        INTO v_oldvalue
        FROM TechnicalManagerData
        WHERE technicalmanager = v_idtm;        
        if v_oldvalue = 0 then
            raise not_exists;
        end if;
        UPDATE TechnicalManagerData
        SET correctExecutions = correctExecutions - 1
         WHERE technicalmanager = v_idtm;
    END IF;
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN not_exists THEN
        rsp := 'ERROR: El TM no t execucions correctes';
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - UPDATEREPLANNEDCHANGES
/**************************************************************************/
CREATE OR REPLACE PROCEDURE UpdateReplannedChanges (
    i_idapp AppInventory.idApp%type,
    i_column varchar,
    i_type varchar,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_exists NUMBER;
    v_newvalue NUMBER;
    v_oldvalue NUMBER;
    wrong_column exception;
    wrong_type exception;
    not_exists exception;
BEGIN    
    rsp := 'OK';
    param := i_idapp || i_column || i_type ; 
    --validem que els parmetres d'entrada siguin correctes
    if i_column != 'Replanned' and i_column != 'Total' then
        raise wrong_column;
    end if;
    if i_type != 'Increase' and i_type != 'Decrease' then
        raise wrong_type;
    end if;
    --iterem sobre les rees geogrfiques de l'aplicaci, per actualitzar les dades de cada una
    FOR r_geoarea IN (
        SELECT 
           geoarea
        FROM 
            geoareaapp
        where idapp = i_idapp
    )
      LOOP
        --comprovem si ja hi ha dades de l'rea a la taula
        select count(*) into v_exists
        from replannedchanges
        where geoarea = r_geoarea.geoarea;
        if v_exists = 0 then
            --si no hi ha dades i es demana reduir el valor, error
            if i_type = 'Decrease' then
                raise not_exists;
            --si no hi ha dades i demana incrementar valor, es crea la lnia.
            elsif i_type = 'Increase' and i_column = 'Replanned' then
                INSERT INTO replannedChanges (
                    geoArea, replanned, total
                ) VALUES (
                    r_geoarea.geoarea, 1, 0 
                );
            elsif i_type = 'Increase' and i_column = 'Total' then
                INSERT INTO replannedChanges (
                    geoArea, replanned, total
                ) VALUES (
                    r_geoarea.geoarea, 0, 1 
                );
            end if;
        --ja hi ha dades a la taula, cal actualitzar els valors
        elsif v_exists > 0 then
            --S'actualitza valor canvis replanificats
            if i_column = 'Replanned' then
                select replanned into v_oldvalue
                from replannedchanges 
                where geoarea = r_geoarea.geoarea;
                if i_type = 'Increase' then
                    v_newvalue := v_oldvalue + 1; 
                elsif i_type = 'Decrease' then
                    if v_oldvalue = 0 then
                        raise not_exists;
                    end if;
                    v_newvalue := v_oldvalue - 1; 
                end if;
                update replannedchanges
                set replanned = v_newvalue
                where geoarea = r_geoarea.geoarea;
            -- s'actualitzen els valors totals
            elsif i_column = 'Total' then
                select total into v_oldvalue
                from replannedchanges 
                where geoarea = r_geoarea.geoarea;
                if i_type = 'Increase' then
                    v_newvalue := v_oldvalue + 1; 
                elsif i_type = 'Decrease' then
                    if v_oldvalue = 0 then
                        raise not_exists;
                    end if;
                    v_newvalue := v_oldvalue - 1; 
                end if;
                update replannedchanges
                set total = v_newvalue
                where geoarea = r_geoarea.geoarea;
            end if;
        end if;
      END LOOP;
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN not_exists THEN
        rsp := 'ERROR: L''rea geogrfica ja t valor 0';
        addlog($$plsql_unit, param, rsp);
    WHEN wrong_column THEN
        rsp := 'ERROR: La columna a modificar ha de ser ''Replanned'' o ''Total''';
        addlog($$plsql_unit, param, rsp);
    WHEN wrong_type then
        rsp := 'ERROR: L''acci ha de ser ''Increase'' o ''Decrease''';
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - EXECUTECHANGE
/**************************************************************************/
CREATE OR REPLACE PROCEDURE executeChange (
    i_idchange changes.idchange%TYPE, i_date executedchanges.dateexecution%type, 
    i_result executedchanges.result%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    CHANGE_NOT_APPROVED exception;
    v_status changes.statuschange%type;
    v_idapp changes.application%type;
    change_executed exception;
    v_isExecuted number;
BEGIN    
    rsp := 'OK';
    param := i_idchange || i_date || i_result; 
    --s'obt l'aplicaci i l'estat del canvi
    select application, statuschange into v_idapp, v_status
    from changes
    where idchange = i_idchange;
    --si el canvi no est aprovat no es pot executar -> error
    if v_status != 'AP' THEN
        RAISE CHANGE_NOT_APPROVED;
    END IF;
    --si el canvi ja s'ha executat correctament -> error
    select count(*) into v_isExecuted
    from executedChanges
    where idchange = i_idchange
      and result = 'Correct';
    if v_isExecuted > 0 then
        raise change_executed;
    end if;
    --insertar canvi
    INSERT INTO executedChanges (
        idchange, dateexecution, result
    ) VALUES (
        i_idchange, i_date, i_result
    );
    --Augmentar el total de canvis executats (ReplannedChanges-total)
     UpdateReplannedChanges(v_idapp, 'Total', 'Increase', rsp);
    --canvi correcte: +1 execucions correctes del TM i -1 canvis en curs del TM
    if i_result = 'Correct' then
        increaseCorrectExecutionsTM(v_idapp, RSP);
        decreaseCurrChangesTM(v_idapp, RSP);
    --amb incidncies: +1 execucions incorrectes
    elsif i_result = 'Incidences' then
        increaseCounters('IE', extract(year from i_date), rsp );
    --replanificat: +1 execucions incorrectes i +1 canvis replanificats
    elsif i_result = 'Replanned' then
        increaseCounters('IE', extract(year from i_date), rsp );      
        --Augmentar el valor de canvis replanificats (replannedchanges-replanned)
        UpdateReplannedChanges(v_idapp, 'Replanned', 'Increase', rsp);
    end if;    
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    when change_executed then
        rsp := 'ERROR: El canvi ja s''ha executat correctament' ;
        addlog($$plsql_unit, param, rsp);
    WHEN CHANGE_NOT_APPROVED THEN
        rsp := 'ERROR: El canvi ha d''estar aprovat' ;
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - MODIFYEXECUTEDCHANGE
/**************************************************************************/
CREATE OR REPLACE PROCEDURE modifyExecutedChange (
    i_idchange changes.idchange%TYPE, i_newdate executedchanges.dateexecution%type, 
    i_newresult executedchanges.result%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_olddate executedchanges.dateexecution%type;
    v_oldresult executedchanges.result%type;
    v_idapp changes.application%type;
BEGIN    
    rsp := 'OK';
    param := i_idchange || i_newdate || i_newresult;  
    --busquem la data i resultats antics de l'execuci
    SELECT DATEEXECUTION, RESULT INTO V_OLDDATE, V_OLDRESULT
    FROM EXECUTEDCHANGES
    WHERE IDCHANGE = I_IDCHANGE;
    --busquem app del canvi
    select application into v_idapp
    from changes
    where idchange = i_idchange;
    --actualitzem execuci amb els valors nous
    update executedChanges 
    set dateexecution = i_newdate,
        result = i_newresult
       where idchange = i_idchange;
    --si l'any ha canviat i ambds resultats antic i nou son incorrectes
    if extract(year from i_newDate) != extract(year from v_olddate) then
        if ( v_oldresult = 'Incidences' or v_oldresult = 'Replanned' ) and 
         ( i_newResult = 'Incidences' or i_newResult = 'Replanned' ) then
         --actualitzar execucions incorrectes per l'any antic (-1) i any nou (+1)
        increaseCounters('IE', extract(year from i_newDate), rsp );
        decreaseCounters('IE', extract(year from v_olddate), rsp );
        end if;
    end if;
    --Execuci canvia de correcta a incidncies
    if v_oldresult = 'Correct' and i_newResult = 'Incidences' then      
        decreaseCorrectExecutionsTM(v_idapp, rsp); --reduir execucions correctes pel TM      
        increaseCounters('IE', extract(year from i_newDate), rsp );--augmentar execucions incorrectes
    --de correcta a replanned
    elsif v_oldresult = 'Correct' and i_newResult = 'Replanned' then
        decreaseCorrectExecutionsTM(v_idapp, rsp); --reduir execucions correctes pel TM 
        increaseCounters('IE', extract(year from i_newDate), rsp );--augmentar execucions incorrectes
        UpdateReplannedChanges(v_idapp, 'Replanned', 'Increase', rsp);--augmentar execucions replanificades
    --de incidncies a correcta
    elsif v_oldresult = 'Incidences' and i_newResult = 'Correct' then
        increaseCorrectExecutionsTM(v_idapp, rsp); --augmentar execucions correctes pel TM 
        decreaseCounters('IE', extract(year from i_newDate), rsp );--reduir execucions incorrectes
    --de incidncies a replanificada
    elsif v_oldresult = 'Incidences' and i_newResult = 'Replanned' then
        UpdateReplannedChanges(v_idapp, 'Replanned', 'Increase', rsp);--augmentar execucions replanificades
    --de replanificada a correcta
    elsif v_oldresult = 'Replanned' and i_newResult = 'Correct' then
        UpdateReplannedChanges(v_idapp, 'Replanned', 'Decrease', rsp);--reduir execucions replanificades
        decreaseCounters('IE', extract(year from i_newDate), rsp );--reduir execucions incorrectes
        increaseCorrectExecutionsTM(v_idapp, rsp); --augmentar execucions correctes pel TM 
    --de replanificada a incidncies
    elsif v_oldresult = 'Replanned' and i_newResult = 'Incidences' then
        UpdateReplannedChanges(v_idapp, 'Replanned', 'Decrease', rsp);--reduir execucions replanificades
    end if;
    
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - DROPEXECUTEDCHANGE
/**************************************************************************/
CREATE OR REPLACE PROCEDURE dropExecutedChange (
    i_idchange changes.idchange%TYPE,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_howmany number;
    exists_action_plan exception;
    v_idapp changes.application%type;
    v_result executedchanges.result%type;
    v_dateexecution executedchanges.dateexecution%type;
BEGIN    
    rsp := 'OK';
    param := i_idchange; 
    --si hi ha un pla d'acci, no es pot eliminar
    select count(*) into v_howmany
    from actionplan
    where executedchange = i_idchange;
    if v_howmany > 0 then
        raise exists_action_plan;
    end if;
    --obtenir l'aplicaci i el resultat del canvi a eliminar
    select application into v_idapp
    from changes
    where idchange = i_idchange;
    select dateexecution, result into v_dateexecution, v_result
    from executedchanges
    where idchange = i_idchange;
    --borrar l'execuci
    delete executedchanges where idchange = i_idchange;
    --actualitzar la informaci del TM i dels canvis replanificats adequadament
    UpdateReplannedChanges(v_idapp, 'Total', 'Decrease', rsp);--reduir execucions totals
    if v_result = 'Correct' THEN
        decreaseCorrectExecutionsTM(v_idapp, rsp); --reduir execucions correctes pel TM 
        increaseCurrChangesTM(v_idapp, rsp); -- augmentar canvis en curs pel TM
    ELSIF V_RESULT = 'Incidences' then
        decreaseCounters('IE', extract(year from v_dateexecution), rsp );--reduir execucions incorrectes
    elsif v_result = 'Replanned' then
        decreaseCounters('IE', extract(year from v_dateexecution), rsp );--reduir execucions incorrectes
        UpdateReplannedChanges(v_idapp, 'Replanned', 'Decrease', rsp);--reduir execucions totals
    end if;
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    when exists_action_plan then
        rsp := 'ERROR: Hi ha un pla d''acci pel canvi executat';
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - CREATEACTIONPLAN
/**************************************************************************/
CREATE OR REPLACE PROCEDURE createActionPlan (
    i_idplan actionplan.idplan%type,
    i_name actionplan.name%type,
    i_description actionplan.description%type,
    i_inidate actionplan.inidate%type,
    i_enddate actionplan.enddate%type,
    i_idchange changes.idchange%TYPE,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_idapp appinventory.idapp%type;
    v_tm membergcab.idmember%type;
    v_cm membergcab.idmember%type;
    v_howmany number;
    change_not_valid exception;
BEGIN    
    rsp := 'OK';
    param := i_idplan || i_name || i_description || i_inidate || i_enddate || i_idchange; 
    --validar que el canvi ha estat executat amb incidncies o replanificat
    select count(*) into v_howmany
    from executedchanges
    where idchange = i_idchange
    and ( result = 'Incidences' or result = 'Replanned' );
    if v_howmany = 0 then
        raise change_not_valid;
    end if;
    --Busquem l'aplicaci del canvi
    select application into v_idapp
    from changes
    where idchange = i_idchange;
    --busquem el responsable tcnic de l'aplicaci
    select technicalResp into v_tm
    from appinventory
    where idapp = v_idapp;
    --busquem el gestor de canvis de l'empresa
    select idmember into v_cm
    from membergcab
    where rolegcab = 'CM';

    INSERT INTO ActionPlan (
            idplan, name, description, isvalidated, inidate, enddate, executedchange, responsible, validator
        ) VALUES (
            i_idplan, i_name, i_description, 'F', i_inidate, i_enddate, i_idchange, v_tm, v_cm
        );
    --incrementar comptador de plans d'acci oberts (OP)
    increaseCounters('OP', '9999', RSP);
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN change_not_valid THEN
        rsp := 'ERROR: El canvi s''ha executat correctament o no s''ha executat';
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - MODIFYACTIONPLAN
/**************************************************************************/
CREATE OR REPLACE PROCEDURE modifyActionPlan (
    i_idplan actionplan.idplan%type,
    i_newname actionplan.name%type,
    i_newdescription actionplan.description%type,
    i_newisvalidated actionplan.isvalidated%type,
    i_newinidate actionplan.inidate%type,
    i_newenddate actionplan.enddate%type,
    i_newchange changes.idchange%TYPE,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_oldchange actionplan.executedchange%type;
    v_howmanyExecutedChanges number;
    v_howmanyActions number;
    v_howmanyOpenActions number;
    v_oldisvalidated actionplan.isvalidated%type;
    v_idapp appinventory.idapp%type;
    change_not_valid exception;
    closed_actions exception;
BEGIN    
    rsp := 'OK';
    param := i_idplan || i_newname || i_newdescription || i_newisvalidated ||
        i_newinidate || i_newenddate || i_newchange; 
    --si el canvi es modifica, comprovar que el nou canvi s'hagi executat amb incidncies o replannificat
    select isvalidated, executedchange into v_oldisvalidated, v_oldchange
    from actionplan
    where idplan = i_idplan;
    if v_oldchange != i_newchange then
        select count(*) into v_howmanyExecutedChanges
        from executedchanges
        where idchange = i_newchange
        and ( result = 'Incidences' or result = 'Replanned' );
        if v_howmanyExecutedChanges = 0 then
            raise change_not_valid;
        end if;
    end if; 
    --Si es tanca un pla d'acci obert
    if v_oldisvalidated = 'F' and i_newisvalidated = 'T' THEN
        -- validar que t accions i que totes estan tancades
        select count(*) into v_howmanyOpenActions
        from action 
        where idplan = i_idplan and dateaction is null;
        if v_howmanyOpenActions = 0 then
            select count(*) into v_howmanyActions
            from action 
            where idplan = i_idplan ;
            --el pla d'acci no t accions, no es pot tancar: error.
            if v_howmanyActions = 0 then
             raise closed_actions;
            end if;
        --hi ha accions obertes: error
        elsif v_howmanyOpenActions > 0 then
            raise closed_actions;
        end if;
        
    end if;
    --obtenir l'aplicaci del canvi
    select application into v_idapp
    from changes where idchange = i_newchange;
--    -- --actualitzar el canvi
    UPDATE ACTIONPLAN
    SET name = i_newname, 
        description = i_newdescription, 
        isvalidated = i_newisvalidated,
        inidate = i_newinidate,
        enddate = i_newenddate,
        executedchange = i_newchange
    WHERE idplan = i_idplan;
    --Si s'obre un pla d'acci tancat, 
    if v_oldisvalidated = 'T' and i_newisvalidated = 'F' THEN    
        increaseCounters('OP', '9999', RSP); --incrementar comptador de plans oberts
        increaseCurrChangesTM(v_idapp, rsp); --incrementar canvis en curs pel TM de l'app
    --si es tanca un pla d'acci obert
    elsif v_oldisvalidated = 'F' and i_newisvalidated = 'T' THEN
        decreaseCounters('OP', '9999', RSP); --reduir plans d'acci oberts -1
        decreaseCurrChangesTM(v_idapp, rsp); --reduir canvis en curs pel TM de l'app
    END IF;
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    when closed_actions then
        rsp := 'ERROR: Hi ha accions obertes o el pla no cont accions';
        addlog($$plsql_unit, param, rsp);
    WHEN change_not_valid THEN
        rsp := 'ERROR: El nou canvi s''ha executat correctament o no s''ha executat';
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - DROPACTIONPLAN
/**************************************************************************/
CREATE OR REPLACE PROCEDURE dropActionPlan (
    i_idplan actionplan.idplan%TYPE,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    exist_action exception;
    plan_closed exception;
    v_howmanyActions number;
    v_isvalidated actionplan.isvalidated%type;
BEGIN    
    rsp := 'OK';
    param := i_idplan; 
    --si el pla est tancat no es pot eliminar: error
    select isvalidated into v_isvalidated
    from actionplan
    where idplan = i_idplan;
    if v_isvalidated = 'T' THEN
        RAISE plan_closed;
    end if;
    --Si el pla t accions no es pot eliminar: error
    select count(*) into v_howmanyActions
    from action 
    where idplan = i_idplan ;
    if v_howmanyActions > 0 then
        raise exist_action;
    end if;
    --s'elimina el pla
    delete from actionplan where idplan = i_idplan;
    --Es redueix el comptador de plans d'acci oberts
    decreaseCounters('OP', '9999', RSP); 
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION    
    WHEN plan_closed THEN
        rsp := 'ERROR: El pla est tancat i no es pot eliminar' ;
        addlog($$plsql_unit, param, rsp);
    WHEN exist_action THEN
        rsp := 'ERROR: El pla cont accions i no es pot eliminar' ;
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;

/**************************************************************************/
/*PROCEDIMENT - UPDATEREACTIONSINTIME
/**************************************************************************/
CREATE OR REPLACE PROCEDURE UpdateActionsInTime (
    i_date action.dateaction%type,
    i_column varchar,
    i_type varchar,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_exists NUMBER;
    v_newvalue NUMBER;
    v_oldvalue NUMBER;
    wrong_column exception;
    wrong_type exception;
    not_exists exception;
    v_year actionsintime.yearaction%type;
BEGIN    
    rsp := 'OK';
    param :=  i_date || i_column || i_type ; 
    v_year := extract(year from i_date);
    --validem que els parmetres d'entrada siguin correctes
    if i_column != 'InTime' and i_column != 'Total' then
        raise wrong_column;
    end if;
    if i_type != 'Increase' and i_type != 'Decrease' then
        raise wrong_type;
    end if;

    --comprovem si ja hi ha dades de l'any a la taula
    select count(*) into v_exists
    from actionsintime
    where yearaction = v_year;
    if v_exists = 0 then
        --si no hi ha dades i es demana reduir el valor, error
        if i_type = 'Decrease' then
            raise not_exists;
        --si no hi ha dades i demana incrementar valor, es crea la lnia.
        elsif i_type = 'Increase' and i_column = 'InTime' then
            INSERT INTO actionsintime (
                yearAction, inTime, total
            ) VALUES (
                v_year, 1, 0
            );
        elsif i_type = 'Increase' and i_column = 'Total' then
            INSERT INTO actionsintime (
                yearAction, inTime, total
            ) VALUES (
                v_year, 0, 1
            );
        end if;
    --Si hi ha dades a la taula, cal actualitzar els valors
    elsif v_exists > 0 then
        --S'actualitza valor  d'accions dins el termini
        if i_column = 'InTime' then
            select intime into v_oldvalue
            from actionsintime 
            where yearaction = v_year;
            if i_type = 'Increase' then
                v_newvalue := v_oldvalue + 1; 
            elsif i_type = 'Decrease' then
                if v_oldvalue = 0 then
                    raise not_exists;
                end if;
                v_newvalue := v_oldvalue - 1; 
            end if;
            update actionsintime
            set intime = v_newvalue
            where yearaction = v_year;
        -- s'actualitzen els valors totals
        elsif i_column = 'Total' then
            select total into v_oldvalue
            from actionsintime 
            where yearaction = v_year;
            if i_type = 'Increase' then
                v_newvalue := v_oldvalue + 1; 
            elsif i_type = 'Decrease' then
                if v_oldvalue = 0 then
                    raise not_exists;
                end if;
                v_newvalue := v_oldvalue - 1; 
            end if;
            update actionsintime
            set total = v_newvalue
            where yearaction = v_year;
        end if;
    end if;

    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN not_exists THEN
        rsp := 'ERROR: L''any ja t valor 0';
        addlog($$plsql_unit, param, rsp);
    WHEN wrong_column THEN
        rsp := 'ERROR: La columna a modificar ha de ser ''InTime'' o ''Total''';
        addlog($$plsql_unit, param, rsp);
    WHEN wrong_type then
        rsp := 'ERROR: L''acci ha de ser ''Increase'' o ''Decrease''';
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - CREATEACTION
/**************************************************************************/
CREATE OR REPLACE PROCEDURE createAction (
    i_idaction action.idaction%type,
    i_idplan action.idplan%type,
    i_name action.name%type,
    i_description action.description%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
BEGIN    
    rsp := 'OK';
    param := i_idaction || i_idplan || i_name || i_description ; 

    INSERT INTO action (
           idaction, idplan, name, description, dateaction
        ) VALUES (
            i_idaction, i_idplan, i_name, i_description, ''
        );

    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION

    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - MODIFYACTION
/**************************************************************************/
CREATE OR REPLACE PROCEDURE modifyAction (
    i_idaction action.idaction%type,
    i_idplan action.idplan%type,
    i_newname action.name%type,
    i_newdescription action.description%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
BEGIN    
    rsp := 'OK';
    param := i_idaction || i_idplan || i_newname || i_newdescription ; 
    -- --actualitzar el canvi
    UPDATE action
    SET name = i_newname,
        description = i_newdescription
    where idaction = i_idaction and idplan = i_idplan;
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;

/**************************************************************************/
/*PROCEDIMENT - MODIFYACTIONDATE
/**************************************************************************/
CREATE OR REPLACE PROCEDURE modifyActionDate (
    i_idaction action.idaction%type,
    i_idplan action.idplan%type,
    i_newDate action.dateaction%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_olddate action.dateaction%type;
    v_inidate actionplan.inidate%type;
    v_enddate actionplan.enddate%type;
    V_newIsInTime ACTIONPLAN.ISVALIDATED%TYPE;
    V_oldIsInTime ACTIONPLAN.ISVALIDATED%TYPE;
BEGIN    
    rsp := 'OK';
    param := i_idaction || i_idplan || i_newdate ; 
    v_newIsInTime := 'F';
    v_oldIsInTime := 'F';
    --busquem la data anterior
    select dateaction into v_olddate
    from action
    where idaction = i_idaction and idplan = i_idplan;
    --busquem el termini del pla d'acci i si les dates anteriors i nova estan dins el termini
    select inidate, enddate into v_inidate, v_enddate
    from actionplan
    where idplan = i_idplan;
    if i_newdate >= v_inidate and i_newdate <= v_enddate then
        v_newIsInTime := 'T';--Nova data dins el termini
    END IF;
    if v_olddate >= v_inidate and v_olddate <= v_enddate then
        v_oldIsInTime := 'T';--data anterior dins el termini
    END IF;
    
    -- actualitzar el canvi
    UPDATE action
    SET dateaction = i_newdate
    where idaction = i_idaction and idplan = i_idplan;
    
    --s'actualitzen els comptadors
    --es tanca l'acci (s'informa la data de l'acci)
    if v_olddate is null then
        --incrementar comptador del total d'accions finalitzades 
        UpdateActionsInTime(i_newDate, 'Total', 'Increase', RSP);
        --l'acci s'ha realitzat dins el termini
        if v_newIsInTime = 'T' then
            --incrementar comptador d'accions finalitzades dins el termini
            UpdateActionsInTime(i_newDate, 'InTime', 'Increase', RSP);
        end if;
    --L'acci ja estava tancada i es modifica la data
    ELSE
        --S'esborra la data (es desf l'acci)
        if i_newDate is null then
            --es disminueix comptador d'accions totals realitzades
            UpdateActionsInTime(v_oldDate, 'Total', 'Decrease', RSP);
            --Si data anterior dins termini: redueix comptador d'accions dins el termini
            if v_oldIsInTime = 'T' THEN
                UpdateActionsInTime(v_oldDate, 'InTime', 'Decrease', RSP);
            END IF;
        --es canvia la data
        elsif v_oldDate != i_newDate and i_newdate is not null then
            --L'acci passa a estar dins el termini: incrementa comptador d'accions dins el termini
            if v_oldIsInTime = 'F' and v_newIsInTime = 'T' THEN
                UpdateActionsInTime(i_newDate, 'InTime', 'Increase', RSP);               
            END IF;
            --L'acci estava dins el termini i passa a estar a fora: reduir comptador d'accions dins el termini
            IF v_oldIsInTime = 'T' and v_newIsInTime = 'F' THEN
                UpdateActionsInTime(v_oldDate, 'InTime', 'Decrease', RSP);
            END IF;
        end if;
    END IF;  

    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - DROPACTION
/**************************************************************************/
CREATE OR REPLACE PROCEDURE dropAction (
    i_idaction action.idaction%TYPE,
    i_idplan action.idplan%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    plan_closed exception;
    v_isvalidated actionplan.isvalidated%type;
    v_enddate actionplan.enddate%type;
    v_inidate actionplan.inidate%type;
    v_dateaction action.dateaction%type;
BEGIN    
    rsp := 'OK';
    param := i_idaction || i_idplan; 
    --si el pla est tancat no es pot eliminar l'acci: error
    select isvalidated, inidate, enddate into v_isvalidated, v_inidate, v_enddate
    from actionplan
    where idplan = i_idplan;
    if v_isvalidated = 'T' THEN
        RAISE plan_closed;
    end if;
    --buscar la data en que es va realitzar l'acci
    select dateaction into v_dateaction
    from action
    where idaction = i_idaction and idplan = i_idplan;
    --eliminar l'acci
    delete from action where idaction = i_idaction and idplan = i_idplan;
    --si l'acci s'havia realitzat
    IF v_dateaction is not null then
        --disminuir comptador accions totals realitzades
        UpdateActionsInTime(v_dateaction, 'Total', 'Decrease', RSP);
        --si l'acci es va realitzar dins el pla
        if v_dateaction >= v_inidate and v_dateaction <= v_enddate then
            --disminur comptador accions dins el termini 
            UpdateActionsInTime(v_dateaction, 'InTime', 'Decrease', RSP);
        end if;
    end if;
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN plan_closed THEN
        rsp := 'ERROR: el pla d''acci est tancat' ;
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - CREATEBREACH
/**************************************************************************/
CREATE OR REPLACE PROCEDURE createBreach (
    i_idbreach breaches.idbreach%type,
    i_name breaches.name%type,
    i_referenceTo breaches.idbreach%type,
    i_description breaches.description%type,
    i_date breaches.dateBreach%type,
    i_result breaches.resultbreach%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
BEGIN    
    rsp := 'OK';
    param := i_idbreach || i_name || i_referenceTo || i_description || i_date || i_result; 
    
    INSERT INTO breaches (
        idbreach, name, referenceTo, description, datebreach, resultbreach
    ) VALUES (
        i_idbreach, i_name, i_referenceTo, i_description, i_date, i_result 
    );

    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - MODIFYBREACH
/**************************************************************************/
CREATE OR REPLACE PROCEDURE modifyBreach (
    i_idbreach breaches.idbreach%type,
    i_newname breaches.name%type,
    i_newreferenceTo breaches.idbreach%type,
    i_newdescription breaches.description%type,
    i_newdate breaches.dateBreach%type,
    i_newresult breaches.resultbreach%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_olddate breaches.datebreach%type;
    v_oldresult breaches.resultbreach%type;
BEGIN    
    rsp := 'OK';
    param := i_idbreach || i_newname || i_newreferenceTo || i_newdescription || i_newdate || i_newresult; 
    UPDATE breaches
    SET name = i_newname,
        referenceTo = i_newReferenceTo,
        description = i_newDescription,
        datebreach = i_newDate,
        resultbreach = i_newResult    
    WHERE idbreach = i_idbreach;

    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
--/**************************************************************************/
--/*PROCEDIMENT - DROPBREACH
--/**************************************************************************/
CREATE OR REPLACE PROCEDURE dropBreach (
    i_idbreach breaches.idbreach%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
BEGIN    
    rsp := 'OK';
    param := i_idbreach; 
    delete from breaches where idbreach = i_idbreach;

    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - UPDATESNUMBERBREACHES
/**************************************************************************/
CREATE OR REPLACE PROCEDURE updatesNumberBreaches (
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_howmany number;
    v_exists number;
BEGIN    
    rsp := 'OK';
    param := sysdate; 
    --comptem el n d'incompliments en els ltims 6 mesos
    select COUNT(*) INTO V_howmany
    from breaches 
    where datebreach > add_months(sysdate, -6)
        and resultbreach = 'KO';
    --guardem el valor global a counters (BR)
    select count(*) into v_exists
    from counters 
    where idcounter = 'BR' and yearinfo = '9999';
    if v_exists = 0 then
        INSERT INTO counters (
        idcounter, yearinfo, totalnumber
    ) VALUES (
        'BR', '9999', v_howmany
    );
    else
        update counters 
            set totalnumber = v_howmany
            where idcounter = 'BR' and yearinfo = '9999';
    end if;

    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - UPDATEUNPLANNEDSUBSTITUTIONS
/**************************************************************************/
CREATE OR REPLACE PROCEDURE updateUnplannedSubstitutions (
    i_idapprover membergcab.idmember%type,
    i_type varchar,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    not_exists exception;
    wrong_type exception;
    v_exists number;
    v_oldvalue unplannedSubstitutions.unplannedsubst%type;
    v_newvalue unplannedSubstitutions.unplannedsubst%type;
BEGIN    
    rsp := 'OK';
    param := i_idapprover || i_type ; 
    if i_type != 'Increase' and i_type != 'Decrease' then
        raise wrong_type;
    end if;
    SELECT COUNT(*) INTO V_EXISTS
    FROM unplannedSubstitutions
    where idapprover = i_idapprover;
    if v_exists = 0 then
        --si no hi ha dades i es demana reduir el valor, error
        if i_type = 'Decrease' then
            raise not_exists;
        else
            INSERT INTO unplannedSubstitutions (
                   idapprover, unplannedsubst
                ) VALUES (
                    i_idapprover, 1
                );
        end if;
    --ja hi ha dades a la taula, cal actualitzar els valors
    elsif v_exists > 0 then
        SELECT unplannedsubst INTO V_oldvalue
        FROM unplannedSubstitutions
        where idapprover = i_idapprover;
        if i_type = 'Increase' then
            v_newvalue := v_oldvalue + 1;
        elsif i_type = 'Decrease' then
            if v_oldvalue = 0 then
                raise not_exists;
            else
                v_newvalue := v_oldvalue - 1;
            end if;
        end if;
        update unplannedsubstitutions
        set unplannedsubst = v_newValue
        where idapprover = i_idapprover;
 
    end if;

    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN not_exists THEN
        rsp := 'ERROR: L''aprovador no t cap substituci';
        addlog($$plsql_unit, param, rsp);
    WHEN wrong_type then
        rsp := 'ERROR: L''acci ha de ser ''Increase'' o ''Decrease''';
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - CREATESUBSTITUTION
/**************************************************************************/
CREATE OR REPLACE PROCEDURE createSubstitution (
    i_idperson membergcab.idmember%type,
    i_inidate substitutions.inidate%type,
    i_isplanned substitutions.isplanned%type,
    i_reason substitutions.reason%type,
    i_enddate substitutions.enddate%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
	v_subst membergcab.substitute%type;
BEGIN    
    rsp := 'OK';
    param := i_idperson || i_inidate || i_isplanned || i_reason || i_enddate ; 
    
	--buscar el substitut
	SELECT substitute into v_subst
	FROM membergcab
	WHERE idmember = i_idperson;
    INSERT INTO substitutions (
        idperson, inidate, substitute, isplanned, reason, enddate
    ) VALUES (
        i_idperson, i_inidate, v_subst, i_isplanned, i_reason, i_enddate 
    );
    --si la substituci no est planificada 
    if i_isplanned = 'F' then 
        -- +1 substitucions no planificades per l'aprovador subsitut
        updateUnplannedSubstitutions(i_idperson, 'Increase', RSP);
    end if;

    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - MODIFYSUBSTITUTION
/**************************************************************************/
CREATE OR REPLACE PROCEDURE modifySubstitution (
    i_idperson membergcab.idmember%type,
    i_inidate substitutions.inidate%type,
    i_newisplanned substitutions.isplanned%type,
    i_newreason substitutions.reason%type,
    i_newenddate substitutions.enddate%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_oldisplanned substitutions.isplanned%type;
BEGIN    
    rsp := 'OK';
    param := i_idperson || i_inidate || i_newisplanned || i_newreason || i_newenddate ; 
    --obtenir valor anterior de isPlanned
    select isplanned into v_oldisplanned
    from substitutions
    where idperson = i_idperson and inidate = i_inidate;
    --actualitzar substituci
    UPDATE substitutions
    set isplanned = i_newisplanned, 
        reason = i_newreason,
        enddate = i_newenddate
    WHERE idperson = i_idperson and inidate = i_inidate;
    if v_oldisplanned != i_newisplanned then
        --es canvia de no planificada a planificada
        if v_oldisplanned = 'F' and i_newisplanned = 'T' THEN
            -- reduir substitucions no planificades per l'aprovador 
            updateUnplannedSubstitutions(i_idperson, 'Decrease', RSP);
        --es canvia de planificada a no planificada
        elsif v_oldisplanned = 'T' and i_newisplanned = 'F' THEN
            -- augmentar substitucions no planificades per l'aprovador 
            updateUnplannedSubstitutions(i_idperson, 'Increase', RSP);
        END IF;
    end if;
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN NO_DATA_FOUND THEN 
        rsp := 'ERROR: La substituci no existeix' ;
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
/**************************************************************************/
/*PROCEDIMENT - DROPSUBSTITUTION
/**************************************************************************/
CREATE OR REPLACE PROCEDURE dropSubstitution (
    i_idperson membergcab.idmember%type,
    i_inidate substitutions.inidate%type,
    rsp OUT VARCHAR
) IS
    param VARCHAR(500);
    v_isplanned substitutions.isplanned%type;
BEGIN    
    rsp := 'OK';
    param := i_idperson || i_inidate ;
    --buscar si s una substituci planificada o no
    select isplanned into v_isplanned
    from substitutions
    where idperson = i_idperson and inidate = i_inidate;
    --esborrar substituci
    delete from substitutions where idperson = i_idperson and inidate = i_inidate;
    --substituci no planificada
    if v_isplanned = 'F' THEN
        -- reduir substitucions no planificades per l'aprovador 
        updateUnplannedSubstitutions(i_idperson, 'Decrease', RSP);
    END IF;
    --registre al log
    addlog($$plsql_unit, param, rsp);
EXCEPTION
    WHEN NO_DATA_FOUND THEN 
        rsp := 'ERROR: La substituci no existeix' ;
        addlog($$plsql_unit, param, rsp);
    WHEN OTHERS THEN
        rsp := 'ERROR' || ':' || sqlerrm;
        addlog($$plsql_unit, param, rsp);
END;
