
CREATE TABLE person ( 
    idPerson VARCHAR2(15) CONSTRAINT PK_Person PRIMARY KEY,
    nif VARCHAR2(10) CONSTRAINT AK_Person UNIQUE NOT NULL,
    NAME VARCHAR2(20) CONSTRAINT NN_PersonName NOT NULL,
    surname VARCHAR2(40) CONSTRAINT NN_PersonSurname NOT NULL,
    birthDate DATE NOT NULL,
    gender VARCHAR2(6) NOT NULL,
    CONSTRAINT gender CHECK (gender IN ('Male', 'Female', 'Other') )
    );
    
CREATE TABLE memberGCAB (
    idMember VARCHAR2(15)  PRIMARY KEY, 
    roleGCAB varchar2(2)  NOT NULL,
    substitute varchar2(15)  NOT NULL,
    CONSTRAINT FK_Member_id FOREIGN KEY (idMember) REFERENCES PERSON(idPerson),
    CONSTRAINT FK_Member_substitute FOREIGN KEY (substitute) REFERENCES PERSON(idPerson),
    CONSTRAINT roleGCAB CHECK (roleGCAB IN ('TM', 'BM', 'CM') )
);

CREATE TABLE changeCategory (
    importance VARCHAR2(1) CONSTRAINT PK_ChangeCategory PRIMARY KEY,
    creationDate DATE CONSTRAINT NN_CreationDate NOT NULL,
    dateIn DATE CONSTRAINT NN_DateIn NOT NULL,
    dateOut DATE,
    CHECK ( dateOut > dateIn )
);

CREATE TABLE AppInventory (
    idApp VARCHAR2(15)  PRIMARY KEY,
    appName VARCHAR2(20)  UNIQUE NOT NULL,
    datePro DATE  NOT NULL,
    technology varchar2(20)  NOT NULL,
    infraestructure varchar2(20)  NOT NULL,
    numUsers INTEGER  NOT NULL ,
    criticality INTEGER  NOT NULL,
    impactOOS varchar2(6) NOT NULL,
    technicalResp VARCHAR(15) NOT NULL,
    businessResp VARCHAR(15) NOT NULL,
    CONSTRAINT criticality CHECK (criticality>= 0 AND criticality <=10),
    CONSTRAINT impactOOS CHECK (impactOOS IN ('High', 'Medium', 'Low')),
    CONSTRAINT FK_App_TR FOREIGN KEY (technicalResp) REFERENCES MEMBERGCAB(idMember),
    CONSTRAINT FK_App_BR FOREIGN KEY (businessResp) REFERENCES MEMBERGCAB(idMember)
)

CREATE TABLE CHANGES (
    idChange VARCHAR2(15) PRIMARY KEY,
    name VARCHAR2(20) NOT NULL,
    description varchar2(60) NOT NULL,
    geoScope VARCHAR2(6) NOT NULL,
    category VARCHAR2(1) NOT NULL,
    application varchar2(15) NOT NULL,
    statusChange varchar2(2) not null,
    CONSTRAINT geoScope CHECK (geoScope IN ('High', 'Medium', 'Low')),
    CONSTRAINT FK_Changes_Category FOREIGN KEY (category) REFERENCES CHANGECATEGORY(importance),
    CONSTRAINT FK_Changes_App FOREIGN KEY (application) REFERENCES APPINVENTORY(idApp),
    constraint statusChange check (statusChange IN ('PA', 'AP', 'RE'))
);

CREATE TABLE EXECUTEDCHANGES (
    idChange VARCHAR2(15) PRIMARY KEY,
    dateExecution DATE NOT NULL,
    result VARCHAR2(10) NOT NULL,
    CONSTRAINT FK_ExecutedChanges_id FOREIGN KEY (idChange) REFERENCES CHANGES(idChange),
     CONSTRAINT result CHECK (result IN ('Correct', 'Incidences', 'Replanned') )
);


CREATE TABLE ACTIONPLAN (
    idPlan VARCHAR2(15) PRIMARY KEY,
    name VARCHAR2(20) NOT NULL,
    description varchar2(60) NOT NULL,
    isValidated VARCHAR2(1) NOT NULL,
    iniDate DATE  NOT NULL,
    endDate DATE NOT NULL,
    executedChange VARCHAR2(15) not null,
    responsible VARCHAR2(15 BYTE) NOT NULL,
    validator VARCHAR2(15 BYTE) NOT NULL,
    CONSTRAINT isValidated CHECK (isValidated IN ('T', 'F') ),
    CHECK ( endDate > iniDate ),
    CONSTRAINT FK_ActionPlan_Change FOREIGN KEY (executedChange) REFERENCES EXECUTEDCHANGES(IDCHANGE),
    CONSTRAINT FK_ActionPlan_Responsible FOREIGN KEY (responsible) REFERENCES MEMBERGCAB(idMember),
    CONSTRAINT FK_ActionPlan_Validator FOREIGN KEY (validator) REFERENCES MEMBERGCAB(idMember)
);

CREATE TABLE ACTION (
    idAction VARCHAR2(15) NOT NULL,
    idPlan VARCHAR2(15) NOT NULL,
    name VARCHAR2(20) NOT NULL,
    description varchar2(60) NOT NULL,
    dateAction date,
    CONSTRAINT PK_Action PRIMARY KEY(idAction, idPlan),
    CONSTRAINT FK_Action_IdPlan FOREIGN KEY (idPlan) REFERENCES ACTIONPLAN(idPlan)
);

CREATE TABLE APPROVAL (
    idPerson VARCHAR2(15) NOT NULL,
    idChange VARCHAR2(15) NOT NULL,
    dateApproval DATE NOT NULL,
    roleApprover varchar2(2)  NOT NULL,
    commentary VARCHAR2(100),
    CONSTRAINT PK_Approval PRIMARY KEY(idPerson, idChange),
    CONSTRAINT FK_Approval_IdPerson FOREIGN KEY (idPerson) REFERENCES PERSON(idperson),
    CONSTRAINT FK_Approval_idChange FOREIGN KEY (idChange) REFERENCES CHANGES(idChange)
    
);

CREATE TABLE SUBSTITUTIONS (
    idPerson VARCHAR2(15) NOT NULL,
    iniDate DATE NOT NULL,
    substitute VARCHAR2(15) NOT NULL,
    isPlanned VARCHAR2(1) NOT NULL,
    reason VARCHAR(60) NOT NULL,
    endDate DATE NOT NULL,
    CONSTRAINT PK_Substitutions PRIMARY KEY(idPerson, iniDate),
    CHECK (endDate > iniDate),
    CONSTRAINT PK_Substitute FOREIGN KEY (substitute) REFERENCES PERSON(idPerson),
    CONSTRAINT PK_Substitute_idPerson FOREIGN KEY (idPerson) REFERENCES MEMBERGCAB(idMember)
);

CREATE TABLE INVITATIONS (
    idHost VARCHAR2(15) NOT NULL,
    idGuest VARCHAR2(15) NOT NULL,
    iniDate DATE NOT NULL,
    reason VARCHAR(60) NOT NULL,
    endDate DATE NOT NULL,
    CHECK (endDate > iniDate),
    CONSTRAINT PK_Invitations PRIMARY KEY(idHost, idGuest, iniDate),
    CONSTRAINT FK_Invitations_IdHost FOREIGN KEY (idHost) REFERENCES MEMBERGCAB(idMember),
    CONSTRAINT FK_Invitations_IdGuest FOREIGN KEY (idGuest) REFERENCES PERSON(idPerson)
);

CREATE TABLE BREACHES (
    idBreach VARCHAR2(15) PRIMARY KEY,
    name VARCHAR2(20) NOT NULL,
    referenceTo VARCHAR2(15) NOT NULL,
    description VARCHAR2(60) NOT NULL,
    dateBreach DATE NOT NULL,
    resultBreach VARCHAR2(2) not null,
    CONSTRAINT resultBreach CHECK (resultBreach IN ('OK', 'KO') )
);

CREATE TABLE GEOAREAAPP (
    idApp VARCHAR2(15) NOT NULL,
    geoArea VARCHAR2(7) NOT NULL,
    CONSTRAINT PK_Geoareaapp PRIMARY KEY(idApp, geoArea),
    CONSTRAINT FK_Geoareaapp_idapp FOREIGN KEY (idApp) REFERENCES AppInventory(IDAPP),
    CONSTRAINT geoArea CHECK (geoArea IN ('Europa', 'America', 'Asia', 'Oceania') )
);

